@@ -12,6 +12,7 @@ from PyQt4.QtGui import *
 from PyQt4.Qsci import QsciScintilla
 
 from tortoisehg.util.i18n import _
+from tortoisehg.util import hglib
 from tortoisehg.hgqt import cmdcore, qtlib, qscilib
 
 def startProgress(topic, status):
@@ -66,7 +67,7 @@ class ProgressMonitor(QWidget):
 
 
 class ThgStatusBar(QStatusBar):
-    linkActivated = pyqtSignal(QString)
+    linkActivated = pyqtSignal(str)
 
     def __init__(self, parent=None):
         QStatusBar.__init__(self, parent)
@@ -80,7 +81,7 @@ class ThgStatusBar(QStatusBar):
         self.setStyleSheet('QStatusBar::item { border: none }')
         self._updateBusyProgress()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def showMessage(self, ustr, error=False):
         self.lbl.setText(ustr)
         if error:
@@ -110,7 +111,7 @@ class ThgStatusBar(QStatusBar):
         for key in keys:
             self._removeProgress(key)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def clearRepoProgress(self, root):
         root = unicode(root)
         keys = [k for k in self.topics if k[0] == root]
@@ -125,7 +126,7 @@ class ThgStatusBar(QStatusBar):
         self._updateBusyProgress()
 
     # TODO: migrate to setProgress() API
-    @pyqtSlot(QString, object, QString, QString, object)
+    @pyqtSlot(str, object, str, str, object)
     def progress(self, topic, pos, item, unit, total, root=None):
         'Progress signal received from repowidget'
         # topic is current operation
@@ -164,7 +165,7 @@ class ThgStatusBar(QStatusBar):
     def setProgress(self, progress):
         self.progress(*progress)
 
-    @pyqtSlot(unicode, cmdcore.ProgressMessage)
+    @pyqtSlot(str, cmdcore.ProgressMessage)
     def setRepoProgress(self, root, progress):
         self.progress(*(progress + (unicode(root),)))
 
@@ -213,7 +214,7 @@ class LogWidget(qscilib.ScintillaCompat):
             # NOTE: self.setMarkerForegroundColor() doesn't take effect,
             # because it's a *Background* marker.
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def appendLog(self, msg, label):
         """Append log text to the last line; scrolls down to there"""
         self.append(msg)
@@ -285,15 +286,12 @@ class InteractiveUiHandler(cmdcore.UiHandler):
             return text
 
     def _getChoiceInput(self):
-        parts = self._prompttext.split('$$')
-        msg = parts[0].rstrip(' ')
-        choices = [p.strip(' ') for p in parts[1:]]
-        resps = [p[p.index('&') + 1].lower() for p in choices]
+        msg, choicepairs = hglib.extractchoices(self._prompttext)
         dlg = QMessageBox(QMessageBox.Question, _('TortoiseHg Prompt'), msg,
                           QMessageBox.NoButton, self._parentWidget())
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
-        for r, t in zip(resps, choices):
+        for r, t in choicepairs:
             button = dlg.addButton(t, QMessageBox.ActionRole)
             button.response = r
             if r == self._promptdefault:
@@ -333,7 +331,7 @@ class CmdSessionControlWidget(QWidget):
     """Helper widget to implement dialog to run Mercurial commands"""
 
     finished = pyqtSignal(int)
-    linkActivated = pyqtSignal(unicode)
+    linkActivated = pyqtSignal(str)
     logVisibilityChanged = pyqtSignal(bool)
 
     # this won't provide commandFinished signal because the client code
