@@ -8,19 +8,20 @@
 import os
 import sys
 import atexit
+import posixpath
 import shutil
 import shlex
 import stat
 import subprocess
 import tempfile
 import re
+import sip
 import weakref
 
-from mercurial.i18n import _ as hggettext
-from mercurial import commands, extensions, error, util
+from mercurial import extensions, error, util
 
 from tortoisehg.util import hglib, paths, editor, terminal
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from hgext.color import _styles
 
 from PyQt4.QtCore import *
@@ -32,6 +33,19 @@ try:
 except ImportError:
     openflags = 0
 
+# largest allowed size for widget, defined in <src/gui/kernel/qwidget.h>
+QWIDGETSIZE_MAX = (1 << 24) - 1
+
+if PYQT_VERSION < 0x40704:
+    # QObject.sender() goes wrong if connection made in Qt layer.  consider
+    # using QSignalMapper for zero-argument slots instead.
+    def senderSafeSlot(*types, **opts):
+        def deco(func):
+            return func
+        return deco
+else:
+    senderSafeSlot = pyqtSlot
+
 tmproot = None
 def gettempdir():
     global tmproot
@@ -53,7 +67,7 @@ def gettempdir():
 def openhelpcontents(url):
     'Open online help, use local CHM file if available'
     if not url.startswith('http'):
-        fullurl = 'http://tortoisehg.org/manual/2.7/' + url
+        fullurl = 'http://tortoisehg.readthedocs.org/en/latest/' + url
         # Use local CHM file if it can be found
         if os.name == 'nt' and paths.bin_path:
             chm = os.path.join(paths.bin_path, 'doc', 'TortoiseHg.chm')
@@ -71,10 +85,10 @@ def openlocalurl(path):
     '''
 
     if isinstance(path, str):
-        path = QString(hglib.tounicode(path))
-    elif isinstance(path, unicode):
-        path = QString(path)
-    if os.name == 'nt' and path.startsWith('\\\\'):
+        path = hglib.tounicode(path)
+    else:
+        path = unicode(path)
+    if os.name == 'nt' and path.startswith('\\\\'):
         # network share, special handling because of qt bug 13359
         # see http://bugreports.qt.nokia.com/browse/QTBUG-13359
         qurl = QUrl()
@@ -195,35 +209,6 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
                 u'%s : %s' % (hglib.tounicode(cmdline),
                               hglib.tounicode(str(e))))
 
-def savefiles(repo, files, rev, parent=None):
-    for curfile in files:
-        wfile = util.localpath(curfile)
-        wfile, ext = os.path.splitext(os.path.basename(wfile))
-        extfilter = [_("All files (*)")]
-        if wfile:
-            filename = "%s@%d%s" % (wfile, rev, ext)
-            if ext:
-                extfilter.insert(0, "*%s" % ext)
-        else:
-            filename = "%s@%d" % (ext, rev)
-
-        result = QFileDialog.getSaveFileName(parent, _("Save file to"),
-                                             hglib.tounicode(filename),
-                                             ";;".join(extfilter))
-        if not result:
-            continue
-        cwd = os.getcwd()
-        try:
-            os.chdir(repo.root)
-            try:
-                commands.cat(repo.ui, repo, curfile, rev=rev,
-                             output=hglib.fromunicode(result))
-            except (util.Abort, IOError), e:
-                QMessageBox.critical(parent, _('Unable to save file'),
-                                     hglib.tounicode(str(e)))
-        finally:
-            os.chdir(cwd)
-
 def openshell(root, reponame, ui=None):
     if not os.path.exists(root):
         WarningMsgBox(
@@ -254,8 +239,12 @@ def openshell(root, reponame, ui=None):
             # cmd.exe in new window, probably because the initial cmd.exe is
             # invoked with SW_HIDE.
             os.chdir(root)
-            fullargs = map(hglib.tounicode, shlex.split(shellcmd))
-            started = QProcess.startDetached(fullargs[0], fullargs[1:])
+            if os.name == 'nt':
+                # can't parse shellcmd in POSIX way
+                started = QProcess.startDetached(hglib.tounicode(shellcmd))
+            else:
+                fullargs = map(hglib.tounicode, shlex.split(shellcmd))
+                started = QProcess.startDetached(fullargs[0], fullargs[1:])
         finally:
             os.chdir(cwd)
         if not started:
@@ -266,7 +255,7 @@ def openshell(root, reponame, ui=None):
                    _('A terminal shell must be configured'))
 
 
-def isdarktheme(palette=None):
+def isDarkTheme(palette=None):
     """True if white-on-black color scheme is preferable"""
     if not palette:
         palette = QApplication.palette()
@@ -294,8 +283,10 @@ _thgstyles = {
    'log.modified': 'black #ffddaa_background',
    'log.added': 'black #aaffaa_background',
    'log.removed': 'black #ffcccc_background',
+   'log.warning': 'black #ffcccc_background',
    'status.deleted': 'red bold',
    'ui.error': 'red bold #ffcccc_background',
+   'ui.warning': 'black bold #ffffaa_background',
    'control': 'black bold #dddddd_background',
 }
 
@@ -349,8 +340,15 @@ def geteffect(labels):
                 effects.append('color: ' + e)
     return ';'.join(effects)
 
-def applyeffects(chars, effects):
-    return '<span style="white-space: pre;%s">%s</span>' % (effects, chars)
+def gettextcoloreffect(labels):
+    """Map labels like "log.date" to foreground color if available"""
+    for l in str(labels).split():
+        if not l:
+            continue
+        for e in _styles.get(l, '').split():
+            if e.startswith('#') or e in QColor.colorNames():
+                return QColor(e)
+    return QColor()
 
 def getbgcoloreffect(labels):
     """Map labels like "log.date" to background color if available
@@ -488,6 +486,13 @@ def descriptionhtmlizer(ui):
 
 _iconcache = {}
 
+if getattr(sys, 'frozen', False) and os.name == 'nt':
+    def iconpath(f, *insidef):
+        return posixpath.join(':/icons', f, *insidef)
+else:
+    def iconpath(f, *insidef):
+        return os.path.join(paths.get_icon_path(), f, *insidef)
+
 if hasattr(QIcon, 'hasThemeIcon'):  # PyQt>=4.7
     def _findthemeicon(name):
         if QIcon.hasThemeIcon(name):
@@ -496,36 +501,32 @@ else:
     def _findthemeicon(name):
         pass
 
-def _findicon(name):
-    # TODO: icons should be placed at single location before release
+def _findcustomicon(name):
+    # let a user set the icon of a custom tool button
     if os.path.isabs(name):
         path = name
         if QFile.exists(path):
             return QIcon(path)
-    else:
-        for pfx in (':/icons', os.path.join(paths.get_icon_path(), 'svg'),
-                    paths.get_icon_path()):
-            for ext in ('svg', 'png', 'ico'):
-                path = '%s/%s.%s' % (pfx, name, ext)
-                if QFile.exists(path):
-                    return QIcon(path)
-
     return None
 
 # http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html
 _SCALABLE_ICON_PATHS = [(QSize(), 'scalable/actions', '.svg'),
                         (QSize(), 'scalable/apps', '.svg'),
                         (QSize(), 'scalable/status', '.svg'),
+                        (QSize(16, 16), '16x16/actions', '.png'),
                         (QSize(16, 16), '16x16/apps', '.png'),
+                        (QSize(16, 16), '16x16/mimetypes', '.png'),
+                        (QSize(16, 16), '16x16/status', '.png'),
                         (QSize(22, 22), '22x22/actions', '.png'),
                         (QSize(32, 32), '32x32/actions', '.png'),
+                        (QSize(32, 32), '32x32/status', '.png'),
                         (QSize(24, 24), '24x24/actions', '.png')]
 
 def getallicons():
     """Get a sorted, unique list of all available icons"""
     iconset = set()
     for size, subdir, sfx in _SCALABLE_ICON_PATHS:
-        path = ':/icons/%s' % (subdir)
+        path = iconpath(subdir)
         d = QDir(path)
         d.setNameFilters(['*%s' % sfx])
         for iconname in d.entryList():
@@ -536,7 +537,7 @@ def _findscalableicon(name):
     """Find icon from qrc by using freedesktop-like icon lookup"""
     o = QIcon()
     for size, subdir, sfx in _SCALABLE_ICON_PATHS:
-        path = ':/icons/%s/%s%s' % (subdir, name, sfx)
+        path = iconpath(subdir, name + sfx)
         if QFile.exists(path):
             for mode in (QIcon.Normal, QIcon.Active):
                 o.addFile(path, size, mode)
@@ -556,8 +557,8 @@ def geticon(name):
     except KeyError:
         _iconcache[name] = (_findthemeicon(name)
                             or _findscalableicon(name)
-                            or _findicon(name)
-                            or QIcon(':/icons/fallback.svg'))
+                            or _findcustomicon(name)
+                            or QIcon())
         return _iconcache[name]
 
 
@@ -595,6 +596,48 @@ def getcheckboxpixmap(state, bgcolor, widget):
     style.drawPrimitive(style.PE_IndicatorCheckBox, option, painter)
     return pix
 
+
+# On machines with a retina display running OSX (i.e. "darwin"), most icons are
+# too big because Qt4 does not support retina displays very well.
+# To fix that we let users force tortoishg to use smaller icons by setting a
+# THG_RETINA environment variable to True (or any value that mercurial parses
+# as True.
+IS_RETINA = False
+if sys.platform == 'darwin':
+    IS_RETINA = util.parsebool(os.environ.get('THG_RETINA', '0'))
+
+
+def _fixIconSizeForRetinaDisplay(s):
+    if IS_RETINA and s > 1:
+        s /= 2
+    return s
+
+def smallIconSize():
+    style = QApplication.style()
+    s = style.pixelMetric(QStyle.PM_SmallIconSize)
+    s = _fixIconSizeForRetinaDisplay(s)
+    return QSize(s, s)
+
+def toolBarIconSize():
+    if sys.platform == 'darwin':
+        # most Mac users will have laptop-sized screens and prefer a smaller
+        # toolbar to preserve vertical space.
+        style = QCommonStyle()
+    else:
+        style = QApplication.style()
+    s = style.pixelMetric(QStyle.PM_ToolBarIconSize)
+    s = _fixIconSizeForRetinaDisplay(s)
+    return QSize(s, s)
+
+def listviewRetinaIconSize():
+    return QSize(16, 16)
+
+def treeviewRetinaIconSize():
+    return QSize(16, 16)
+
+def barRetinaIconSize():
+    return QSize(10, 10)
+
 class ThgFont(QObject):
     changed = pyqtSignal(QFont)
     def __init__(self, name):
@@ -693,8 +736,6 @@ class ChoicePrompt(QDialog):
     def __init__(self, title, message, parent, choices, default=None,
                  esc=None, files=None):
         QDialog.__init__(self, parent)
-
-        self.setWindowIcon(geticon('thg_logo'))
         self.setWindowTitle(hglib.tounicode(title))
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
 
@@ -746,6 +787,82 @@ def allowCaseChangingInput(combo):
     assert isinstance(combo, QComboBox) and combo.isEditable()
     combo.completer().setCaseSensitivity(Qt.CaseSensitive)
 
+class BadCompletionBlocker(QObject):
+    """Disable unexpected inline completion by enter key if selectAll()-ed
+
+    If the selection state looks in the middle of the completion, QComboBox
+    replaces the edit text by the current completion on enter key pressed.
+    This is wrong in the following scenario:
+
+    >>> combo = QComboBox(editable=True)
+    >>> combo.addItem('history value')
+    >>> combo.setEditText('initial value')
+    >>> combo.lineEdit().selectAll()
+    >>> QApplication.sendEvent(
+    ...     combo, QKeyEvent(QEvent.KeyPress, Qt.Key_Enter, Qt.NoModifier))
+    True
+    >>> str(combo.currentText())
+    'history value'
+
+    In this example, QLineControl picks the first item in the combo box
+    because the completion prefix has not been set.
+
+    BadCompletionBlocker is intended to work around this problem.
+
+    >>> combo.installEventFilter(BadCompletionBlocker(combo))
+    >>> combo.setEditText('initial value')
+    >>> combo.lineEdit().selectAll()
+    >>> QApplication.sendEvent(
+    ...     combo, QKeyEvent(QEvent.KeyPress, Qt.Key_Enter, Qt.NoModifier))
+    True
+    >>> str(combo.currentText())
+    'initial value'
+
+    For details, read QLineControl::processKeyEvent() and complete() of
+    src/gui/widgets/qlinecontrol.cpp.
+    """
+
+    def __init__(self, parent):
+        super(BadCompletionBlocker, self).__init__(parent)
+        if not isinstance(parent, QComboBox):
+            raise ValueError('invalid object to watch: %r' % parent)
+
+    def eventFilter(self, watched, event):
+        if watched is not self.parent():
+            return super(BadCompletionBlocker, self).eventFilter(watched, event)
+        if (event.type() != QEvent.KeyPress
+            or event.key() not in (Qt.Key_Enter, Qt.Key_Return)
+            or not watched.isEditable()):
+            return False
+        # deselect without completion if all text selected
+        le = watched.lineEdit()
+        if le.selectedText() == le.text():
+            le.deselect()
+        return False
+
+class ActionPushButton(QPushButton):
+    """Button which properties are defined by QAction like QToolButton"""
+
+    def __init__(self, action, parent=None):
+        super(ActionPushButton, self).__init__(parent)
+        self.setAutoDefault(False)  # action won't be used as dialog default
+        self._defaultAction = action
+        self.addAction(action)
+        self.clicked.connect(action.trigger)
+        self._copyActionProps()
+
+    def actionEvent(self, event):
+        if (event.type() == QEvent.ActionChanged
+            and event.action() is self._defaultAction):
+            self._copyActionProps()
+        super(ActionPushButton, self).actionEvent(event)
+
+    def _copyActionProps(self):
+        action = self._defaultAction
+        self.setEnabled(action.isEnabled())
+        self.setText(action.text())
+        self.setToolTip(action.toolTip())
+
 class PMButton(QPushButton):
     """Toggle button with plus/minus icon images"""
 
@@ -758,8 +875,8 @@ class PMButton(QPushButton):
         self.setFlat(True)
         self.setAutoDefault(False)
 
-        self.plus = geticon('expander-open')
-        self.minus = geticon('expander-close')
+        self.plus = QIcon(iconpath('expander-open.png'))
+        self.minus = QIcon(iconpath('expander-close.png'))
         icon = expanded and self.minus or self.plus
         self.setIcon(icon)
 
@@ -832,6 +949,8 @@ class StatusLabel(QWidget):
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
+        # same policy as status bar of QMainWindow
+        self.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Fixed)
 
         box = QHBoxLayout()
         box.setContentsMargins(*(0,)*4)
@@ -872,7 +991,7 @@ class StatusLabel(QWidget):
                 icon = geticon(icon)
             elif not isinstance(icon, QIcon):
                 raise TypeError, '%s: bool, str or QIcon' % type(icon)
-            self.status_icon.setShown(True)
+            self.status_icon.setVisible(True)
             self.status_icon.setPixmap(icon.pixmap(16, 16))
 
     def clear_icon(self):
@@ -898,199 +1017,6 @@ class LabeledSeparator(QWidget):
 
         self.setLayout(box)
 
-# Strings and regexes used to convert hashes and subrepo paths into links
-_hashregex = re.compile(r'\b[0-9a-fA-F]{12,}')
-# Currently converting subrepo paths into links only works in English
-_subrepoindicatorpattern = hglib.tounicode(hggettext('(in subrepo %s)') + '\n')
-
-def _linkifyHash(message, subrepo=''):
-    if subrepo:
-        p = 'repo:%s?' % subrepo
-    else:
-        p = 'cset:'
-    replaceexpr = lambda m: '<a href="%s">%s</a>' % (p + m.group(0), m.group(0))
-    return _hashregex.sub(replaceexpr, message)
-
-def _linkifySubrepoRef(message, subrepo, hash=''):
-    if hash:
-        hash = '?' + hash
-    subrepolink = '<a href="repo:%s%s">%s</a>' % (subrepo, hash, subrepo)
-    subrepoindicator = _subrepoindicatorpattern % subrepo
-    linkifiedsubrepoindicator = _subrepoindicatorpattern % subrepolink
-    message = message.replace(subrepoindicator, linkifiedsubrepoindicator)
-    return message
-
-def linkifyMessage(message, subrepo=None):
-    r"""Convert revision id hashes and subrepo paths in messages into links
-
-    >>> linkifyMessage('abort: 0123456789ab!\nhint: foo\n')
-    u'abort: <a href="cset:0123456789ab">0123456789ab</a>!<br>hint: foo<br>'
-    >>> linkifyMessage('abort: foo (in subrepo bar)\n', subrepo='bar')
-    u'abort: foo (in subrepo <a href="repo:bar">bar</a>)<br>'
-    >>> linkifyMessage('abort: 0123456789ab! (in subrepo bar)\nhint: foo\n',
-    ...                subrepo='bar') #doctest: +NORMALIZE_WHITESPACE
-    u'abort: <a href="repo:bar?0123456789ab">0123456789ab</a>!
-    (in subrepo <a href="repo:bar?0123456789ab">bar</a>)<br>hint: foo<br>'
-
-    subrepo name containing regexp backreference, \g:
-
-    >>> linkifyMessage('abort: 0123456789ab! (in subrepo foo\\goo)\n',
-    ...                subrepo='foo\\goo') #doctest: +NORMALIZE_WHITESPACE
-    u'abort: <a href="repo:foo\\goo?0123456789ab">0123456789ab</a>!
-    (in subrepo <a href="repo:foo\\goo?0123456789ab">foo\\goo</a>)<br>'
-    """
-    message = unicode(message)
-    message = _linkifyHash(message, subrepo)
-    if subrepo:
-        hash = ''
-        m = _hashregex.search(message)
-        if m:
-            hash = m.group(0)
-        message = _linkifySubrepoRef(message, subrepo, hash)
-    return message.replace('\n', '<br>')
-
-class InfoBar(QFrame):
-    """Non-modal confirmation/alert (like web flash or Chrome's InfoBar)
-
-    Layout::
-
-        |widgets ...                |right widgets ...|x|
-    """
-    finished = pyqtSignal(int)  # mimic QDialog
-    linkActivated = pyqtSignal(unicode)
-
-    # type of InfoBar (the number denotes its priority)
-    INFO = 1
-    ERROR = 2
-    CONFIRM = 3
-
-    infobartype = INFO
-
-    _colormap = {
-        INFO: '#e7f9e0',
-        ERROR: '#f9d8d8',
-        CONFIRM: '#fae9b3',
-        }
-
-    def __init__(self, parent=None):
-        super(InfoBar, self).__init__(parent, frameShape=QFrame.StyledPanel,
-                                      frameShadow=QFrame.Plain)
-        self.setAutoFillBackground(True)
-        p = self.palette()
-        p.setColor(QPalette.Window, QColor(self._colormap[self.infobartype]))
-        p.setColor(QPalette.WindowText, QColor("black"))
-        self.setPalette(p)
-
-        self.setLayout(QHBoxLayout())
-        self.layout().setContentsMargins(2, 2, 2, 2)
-
-        self.layout().addStretch()
-        self._closebutton = QPushButton(self, flat=True, autoDefault=False,
-            icon=self.style().standardIcon(QStyle.SP_DockWidgetCloseButton))
-        self._closebutton.clicked.connect(self.close)
-        self.layout().addWidget(self._closebutton)
-
-    def addWidget(self, w, stretch=0):
-        self.layout().insertWidget(self.layout().count() - 2, w, stretch)
-
-    def addRightWidget(self, w):
-        self.layout().insertWidget(self.layout().count() - 1, w)
-
-    def closeEvent(self, event):
-        if self.isVisible():
-            self.finished.emit(0)
-        super(InfoBar, self).closeEvent(event)
-
-    def keyPressEvent(self, event):
-        if event.key() == Qt.Key_Escape:
-            self.close()
-        super(InfoBar, self).keyPressEvent(event)
-
-    def heightForWidth(self, width):
-        # loosely based on the internal strategy of QBoxLayout
-        if self.layout().hasHeightForWidth():
-            return super(InfoBar, self).heightForWidth(width)
-        else:
-            return self.sizeHint().height()
-
-class StatusInfoBar(InfoBar):
-    """Show status message"""
-    def __init__(self, message, parent=None):
-        super(StatusInfoBar, self).__init__(parent)
-        self._msglabel = QLabel(message, self,
-                                wordWrap=True,
-                                textInteractionFlags=Qt.TextSelectableByMouse \
-                                | Qt.LinksAccessibleByMouse)
-        self._msglabel.linkActivated.connect(self.linkActivated)
-        self.addWidget(self._msglabel, stretch=1)
-
-class CommandErrorInfoBar(InfoBar):
-    """Show command execution failure (with link to open log window)"""
-    infobartype = InfoBar.ERROR
-
-    def __init__(self, message, parent=None):
-        super(CommandErrorInfoBar, self).__init__(parent)
-
-        self._msglabel = QLabel(message, self,
-                                wordWrap=True,
-                                textInteractionFlags=Qt.TextSelectableByMouse \
-                                | Qt.LinksAccessibleByMouse)
-        self._msglabel.linkActivated.connect(self.linkActivated)
-        self.addWidget(self._msglabel, stretch=1)
-
-        self._loglabel = QLabel('<a href="log:">%s</a>' % _('Show Log'))
-        self._loglabel.linkActivated.connect(self.linkActivated)
-        self.addRightWidget(self._loglabel)
-
-class ConfirmInfoBar(InfoBar):
-    """Show confirmation message with accept/reject buttons"""
-    accepted = pyqtSignal()
-    rejected = pyqtSignal()
-    infobartype = InfoBar.CONFIRM
-
-    def __init__(self, message, parent=None):
-        super(ConfirmInfoBar, self).__init__(parent)
-
-        # no wordWrap=True and stretch=1, which inserts unwanted space
-        # between _msglabel and _buttons.
-        self._msglabel = QLabel(message, self,
-                                textInteractionFlags=Qt.TextSelectableByMouse \
-                                | Qt.LinksAccessibleByMouse)
-        self._msglabel.linkActivated.connect(self.linkActivated)
-        self.addWidget(self._msglabel)
-
-        self._buttons = QDialogButtonBox(self)
-        self._buttons.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
-        self.acceptButton = self._buttons.addButton(QDialogButtonBox.Ok)
-        self.rejectButton = self._buttons.addButton(QDialogButtonBox.Cancel)
-        self._buttons.accepted.connect(self._accept)
-        self._buttons.rejected.connect(self._reject)
-        self.addWidget(self._buttons)
-
-        # so that acceptButton gets focus by default
-        self.setFocusProxy(self._buttons)
-
-    def closeEvent(self, event):
-        if self.isVisible():
-            self.finished.emit(1)
-            self.rejected.emit()
-            self.hide()  # avoid double emission of finished signal
-        super(ConfirmInfoBar, self).closeEvent(event)
-
-    @pyqtSlot()
-    def _accept(self):
-        self.finished.emit(0)
-        self.accepted.emit()
-        self.hide()
-        self.close()
-
-    @pyqtSlot()
-    def _reject(self):
-        self.finished.emit(1)
-        self.rejected.emit()
-        self.hide()
-        self.close()
-
 class WidgetGroups(object):
     """ Support for bulk-updating properties of Qt widgets """
 
@@ -1152,11 +1078,15 @@ class DialogKeeper(QObject):
     >>> dialogs.count()
     1
 
-    closed dialog will be disowned:
+    closed dialog will be deleted:
+
+    >>> def processDeferredDeletion():
+    ...     loop = QEventLoop()
+    ...     QTimer.singleShot(0, loop.quit)
+    ...     loop.exec_()
 
     >>> dlg1.reject()
-    >>> dlg1.parent() is None
-    True
+    >>> processDeferredDeletion()
     >>> dialogs.count()
     0
 
@@ -1179,6 +1109,7 @@ class DialogKeeper(QObject):
     >>> dialogs.open(QDialog) is dlg4
     True
     >>> dlg4.reject()
+    >>> processDeferredDeletion()
     >>> dialogs.count()
     1
     >>> dialogs.open(QDialog) is dlg3
@@ -1189,33 +1120,6 @@ class DialogKeeper(QObject):
 
         self._dialogs = DialogKeeper(self._createDialog)
         self._dialogs = DialogKeeper(lambda *args: Foo(self))
-
-    When to delete reference:
-
-    If a dialog is not referenced, and if accept(), reject() and done() are
-    not overridden, it could be garbage-collected during finished signal and
-    lead to hard crash::
-
-        #0  isSignalConnected (signal_index=..., this=0x0)
-        #1  QMetaObject::activate (...)
-        #2  ... in sipQDialog::done (this=0x1d655b0, ...)
-        #3  ... in sipQDialog::reject (this=0x1d655b0)
-        #4  ... in QDialog::closeEvent (this=this@entry=0x1d655b0, ...)
-
-    To avoid crash, a finished dialog is referenced explicitly by DialogKeeper
-    until next event processing.
-
-    >>> dialogs = DialogKeeper(QDialog)
-    >>> dlgref = weakref.ref(dialogs.open())
-    >>> dialogs.count()
-    1
-    >>> esckeyev = QKeyEvent(QEvent.KeyPress, Qt.Key_Escape, Qt.NoModifier)
-    >>> QApplication.postEvent(dlgref(), esckeyev)  # close without incref
-    >>> QApplication.processEvents()  # should not crash
-    >>> dialogs.count()
-    0
-    >>> dlgref() is None  # nobody should have reference
-    True
     """
 
     def __init__(self, createdlg, genkey=None, parent=None):
@@ -1223,11 +1127,6 @@ class DialogKeeper(QObject):
         self._createdlg = createdlg
         self._genkey = genkey or DialogKeeper._defaultgenkey
         self._keytodlgs = {}  # key: [dlg, ...]
-        self._dlgtokey = {}   # dlg: key
-
-        self._garbagedlgs = []
-        self._emptygarbagelater = QTimer(self, interval=0, singleShot=True)
-        self._emptygarbagelater.timeout.connect(self._emptygarbage)
 
     def open(self, *args, **kwargs):
         """Create new dialog or reactivate existing dialog"""
@@ -1259,33 +1158,22 @@ class DialogKeeper(QObject):
         if key not in self._keytodlgs:
             self._keytodlgs[key] = []
         self._keytodlgs[key].append(dlg)
-        self._dlgtokey[dlg] = key
-        dlg.finished.connect(self._forgetdlg)
+        dlg.setAttribute(Qt.WA_DeleteOnClose)
+        dlg.destroyed.connect(self._cleanupdlgs)
         return dlg
 
-    #@pyqtSlot()
-    def _forgetdlg(self):
-        dlg = self.sender()
-        dlg.finished.disconnect(self._forgetdlg)
-        if dlg.parent() is self.parent():
-            dlg.setParent(None)  # assist gc
-        key = self._dlgtokey.pop(dlg)
-        self._keytodlgs[key].remove(dlg)
-        if not self._keytodlgs[key]:
-            del self._keytodlgs[key]
-
-        # avoid deletion inside finished signal
-        self._garbagedlgs.append(dlg)
-        self._emptygarbagelater.start()
-
+    # "destroyed" is emitted soon after Python wrapper is deleted
     @pyqtSlot()
-    def _emptygarbage(self):
-        del self._garbagedlgs[:]
+    def _cleanupdlgs(self):
+        for key, dialogs in self._keytodlgs.items():
+            livedialogs = [dlg for dlg in dialogs if not sip.isdeleted(dlg)]
+            if livedialogs:
+                self._keytodlgs[key] = livedialogs
+            else:
+                del self._keytodlgs[key]
 
     def count(self):
-        assert len(self._dlgtokey) == sum(len(dlgs) for dlgs
-                                          in self._keytodlgs.itervalues())
-        return len(self._dlgtokey)
+        return sum(len(dlgs) for dlgs in self._keytodlgs.itervalues())
 
     @staticmethod
     def _defaultgenkey(_parent, *args, **_kwargs):
@@ -1299,6 +1187,9 @@ class TaskWidget(object):
     def canExit(self):
         return True
 
+    def reload(self):
+        pass
+
 class DemandWidget(QWidget):
     'Create a widget the first time it is shown'
 
@@ -1360,7 +1251,8 @@ class Spacer(QWidget):
 def _configuredusername(ui):
     # need to check the existence before calling ui.username(); otherwise it
     # may fall back to the system default.
-    if (not os.environ.get('HGUSER') and not ui.config('ui', 'username')
+    if (not os.environ.get('HGUSER')
+        and not ui.config('ui', ['username', 'user'])
         and not os.environ.get('EMAIL')):
         return None
     try:
@@ -1458,7 +1350,7 @@ class PaletteSwitcher(object):
     def __init__(self, targetwidget):
         self._targetwref = weakref.ref(targetwidget)  # avoid circular ref
         self._defaultpalette = targetwidget.palette()
-        if not isdarktheme(self._defaultpalette):
+        if not isDarkTheme(self._defaultpalette):
             filterbgcolor = QColor('#FFFFB7')
         else:
             filterbgcolor = QColor('darkgrey')
