@@ -30,7 +30,7 @@ def _firstword(query):
         pass
 
 def _querytype(repo, query):
-    """
+    r"""
     >>> repo = set('0 1 2 3 . stable'.split())
     >>> _querytype(repo, u'') is None
     True
@@ -48,12 +48,17 @@ def _querytype(repo, query):
     'keyword'
     >>> _querytype(repo, u'tagged()')
     'revset'
+    >>> _querytype(repo, u'\u3000')  # UnicodeEncodeError
+    'revset'
     """
     if not query:
         return
     if '(' in query:
         return 'revset'
-    changeid = _firstword(query)
+    try:
+        changeid = _firstword(query)
+    except UnicodeEncodeError:
+        return 'revset'  # avoid further error on formatspec()
     if not changeid:
         return 'keyword'
     try:
@@ -66,10 +71,10 @@ def _querytype(repo, query):
 class RepoFilterBar(QToolBar):
     """Toolbar for RepoWidget to filter changesets"""
 
-    setRevisionSet = pyqtSignal(unicode)
+    setRevisionSet = pyqtSignal(str)
     filterToggled = pyqtSignal(bool)
 
-    branchChanged = pyqtSignal(unicode, bool)
+    branchChanged = pyqtSignal(str, bool)
     """Emitted (branch, allparents) when branch selection changed"""
 
     showHiddenChanged = pyqtSignal(bool)
@@ -118,7 +123,7 @@ class RepoFilterBar(QToolBar):
         le.installEventFilter(self)
 
         self.clearBtn = QToolButton(self)
-        self.clearBtn.setIcon(qtlib.geticon('filedelete'))
+        self.clearBtn.setIcon(qtlib.geticon('hg-remove'))
         self.clearBtn.setToolTip(_('Clear current query and query text'))
         self.clearBtn.clicked.connect(self.onClearButtonClicked)
         self.addWidget(self.clearBtn)
@@ -181,10 +186,12 @@ class RepoFilterBar(QToolBar):
     def onClearButtonClicked(self):
         if self.revsetcombo.currentText():
             self.revsetcombo.clearEditText()
-            self.runQuery()
         elif not isinstance(self.parentWidget(), QMainWindow):
             # act as "close" button because this isn't managed as toolbar
             self.hide()
+        # always request to clear filter; model may still be filtered even
+        # if edit box is empty
+        self.runQuery()
 
     def selectionChanged(self):
         selection = self.revsetcombo.lineEdit().selectedText()
@@ -222,7 +229,7 @@ class RepoFilterBar(QToolBar):
         self.entrydlg.entry.setText(query)
         self.entrydlg.entry.setCursorPosition(0, len(query))
         self.entrydlg.entry.setFocus()
-        self.entrydlg.setShown(True)
+        self.entrydlg.setVisible(True)
 
     def queryIssued(self, query):
         self.revsetcombo.setEditText(query)
@@ -388,38 +395,36 @@ class RepoFilterBar(QToolBar):
         priomap = {self._repo.dirstate.branch(): -2, 'default': -1}
         branches = sorted(branches, key=lambda e: priomap.get(e, 0))
 
+        branches = map(hglib.tounicode, branches)
+
         self._branchCombo.blockSignals(True)
         self._branchCombo.clear()
         self._branchCombo.addItem(self._allBranchesLabel)
-        for branch in branches:
-            self._branchCombo.addItem(hglib.tounicode(branch))
-            self._branchCombo.setItemData(self._branchCombo.count() - 1,
-                                          hglib.tounicode(branch),
-                                          Qt.ToolTipRole)
+        self._branchCombo.setItemData(self._branchCombo.count() - 1, '')
+        for i, branch in enumerate(branches, self._branchCombo.count()):
+            self._branchCombo.addItem(branch)
+            self._branchCombo.setItemData(i, branch, Qt.ToolTipRole)
+            self._branchCombo.setItemData(i, branch)
         self._branchCombo.setEnabled(self.filterEnabled and bool(branches))
         self._branchCombo.blockSignals(False)
 
-        if curbranch and curbranch not in branches:
+        if not self.setBranch(curbranch):
             self._emitBranchChanged()  # falls back to "show all"
-        else:
-            self.setBranch(curbranch)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def setBranch(self, branch):
         """Change the current branch by name [unicode]"""
-        if not branch:
-            index = 0
-        else:
-            index = self._branchCombo.findText(branch)
+        index = self._branchCombo.findData(branch)
         if index >= 0:
             self._branchCombo.setCurrentIndex(index)
+            return True
+        return False
 
     def branch(self):
         """Return the current branch name [unicode]"""
-        if self._branchCombo.currentIndex() == 0:
-            return ''
-        else:
-            return unicode(self._branchCombo.currentText())
+        index = self._branchCombo.currentIndex()
+        branch = self._branchCombo.itemData(index).toString()
+        return unicode(branch)
 
     def branchAncestorsIncluded(self):
         return self._allparAction.isChecked()
